# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

from fintecture.fintecture_object import FintectureObject


class UsageRecordSummary(FintectureObject):
    OBJECT_NAME = "usage_record_summary"
